@@ -47,6 +47,7 @@ from transformers import (
 from simpletransformers.classification.classification_utils import InputExample, convert_examples_to_features
 from simpletransformers.config.global_args import global_args
 from simpletransformers.config.model_args import ConvAIArgs
+from simpletransformers.config.utils import sweep_config_to_sweep_values
 from simpletransformers.conv_ai.conv_ai_utils import get_dataset
 
 try:
@@ -100,7 +101,7 @@ class ConvAIModel:
 
         if "sweep_config" in kwargs:
             sweep_config = kwargs.pop("sweep_config")
-            sweep_values = {key: value["value"] for key, value in sweep_config.as_dict().items() if key != "_wandb"}
+            sweep_values = sweep_config_to_sweep_values(sweep_config)
             self.args.update_from_dict(sweep_values)
 
         if self.args.manual_seed:
@@ -179,7 +180,8 @@ class ConvAIModel:
                 If not given when evaluate_during_training is enabled, the evaluation data from PERSONA-CHAT will be used.
             **kwargs:
         Returns:
-            None
+            global_step: Number of global steps trained
+            training_details: Average training loss if evaluate_during_training is False or full training progress scores if evaluate_during_training is True
         """  # noqa: ignore flake8"
 
         if args:
@@ -212,7 +214,7 @@ class ConvAIModel:
 
         os.makedirs(output_dir, exist_ok=True)
 
-        global_step, tr_loss = self.train(
+        global_step, training_details = self.train(
             train_dataloader,
             output_dir,
             show_running_loss=show_running_loss,
@@ -309,6 +311,7 @@ class ConvAIModel:
             model = torch.nn.DataParallel(model)
 
         global_step = 0
+        training_progress_scores = None
         tr_loss, logging_loss = 0.0, 0.0
         model.zero_grad()
         train_iterator = trange(int(args.num_train_epochs), desc="Epoch", disable=args.silent)
@@ -466,7 +469,12 @@ class ConvAIModel:
                                             logger.info(f" Patience of {args.early_stopping_patience} steps reached")
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
-                                        return global_step, tr_loss / global_step
+                                        return (
+                                            global_step,
+                                            tr_loss / global_step
+                                            if not self.args.evaluate_during_training
+                                            else training_progress_scores,
+                                        )
                         else:
                             if results[args.early_stopping_metric] - best_eval_metric > args.early_stopping_delta:
                                 best_eval_metric = results[args.early_stopping_metric]
@@ -485,7 +493,12 @@ class ConvAIModel:
                                             logger.info(f" Patience of {args.early_stopping_patience} steps reached")
                                             logger.info(" Training terminated.")
                                             train_iterator.close()
-                                        return global_step, tr_loss / global_step
+                                        return (
+                                            global_step,
+                                            tr_loss / global_step
+                                            if not self.args.evaluate_during_training
+                                            else training_progress_scores,
+                                        )
 
             epoch_number += 1
             output_dir_current = os.path.join(output_dir, "checkpoint-{}-epoch-{}".format(global_step, epoch_number))
@@ -527,7 +540,10 @@ class ConvAIModel:
                         self.save_model(args.best_model_dir, model=model, results=results)
                         early_stopping_counter = 0
 
-        return global_step, tr_loss / global_step
+        return (
+            global_step,
+            tr_loss / global_step if not self.args.evaluate_during_training else training_progress_scores,
+        )
 
     def eval_model(self, eval_file=None, output_dir=None, verbose=True, silent=False, **kwargs):
         """
@@ -788,7 +804,7 @@ class ConvAIModel:
                     out_ids = self.sample_sequence(personality, history, tokenizer, model, args)
             history.append(out_ids)
             history = history[-(2 * args.max_history + 1) :]
-            out_text = tokenizer.decode(out_ids, skip_special_tokens=True)
+            out_text = tokenizer.decode(out_ids, skip_special_tokens=self.args.skip_special_tokens)
             print(out_text)
 
     def interact_single(self, message, history, personality=None, encode_history=True):
@@ -842,7 +858,7 @@ class ConvAIModel:
                     out_ids = self.sample_sequence(personality, history, tokenizer, model, args)
             else:
                 out_ids = self.sample_sequence(personality, history, tokenizer, model, args)
-        out_text = tokenizer.decode(out_ids, skip_special_tokens=True)
+        out_text = tokenizer.decode(out_ids, skip_special_tokens=self.args.skip_special_tokens)
 
         if encode_history:
             raw_history.append(out_text)
