@@ -25,6 +25,7 @@ from multiprocessing import Pool, cpu_count
 
 import pandas as pd
 import torch
+from torch.functional import split
 from torch.nn import CrossEntropyLoss
 from torch.utils.data import Dataset
 from tqdm.auto import tqdm
@@ -33,30 +34,37 @@ from tqdm.auto import tqdm
 class InputExample(object):
     """A single training/test example for token classification."""
 
-    def __init__(self, guid, words, labels):
+    def __init__(self, guid, words, labels, x0=None, y0=None, x1=None, y1=None):
         """Constructs a InputExample.
         Args:
             guid: Unique id for the example.
             words: list. The words of the sequence.
             labels: (Optional) list. The labels for each word of the sequence. This should be
             specified for train and dev examples, but not for test examples.
+            bbox: (Optional) list. The bounding boxes for each word of the sequence.
         """
         self.guid = guid
         self.words = words
         self.labels = labels
+        if x0 is None:
+            self.bboxes = None
+        else:
+            self.bboxes = [[a, b, c, d] for a, b, c, d in zip(x0, y0, x1, y1)]
 
 
 class InputFeatures(object):
     """A single set of features of data."""
 
-    def __init__(self, input_ids, input_mask, segment_ids, label_ids):
+    def __init__(self, input_ids, input_mask, segment_ids, label_ids, bboxes=None):
         self.input_ids = input_ids
         self.input_mask = input_mask
         self.segment_ids = segment_ids
         self.label_ids = label_ids
+        if bboxes:
+            self.bboxes = bboxes
 
 
-def read_examples_from_file(data_file, mode):
+def read_examples_from_file(data_file, mode, bbox=False):
     file_path = data_file
     guid_index = 1
     examples = []
@@ -64,30 +72,87 @@ def read_examples_from_file(data_file, mode):
         words = []
         labels = []
         for line in f:
-            if line.startswith("-DOCSTART-") or line == "" or line == "\n":
-                if words:
-                    examples.append(InputExample(guid="{}-{}".format(mode, guid_index), words=words, labels=labels,))
-                    guid_index += 1
-                    words = []
-                    labels = []
-            else:
-                splits = line.split(" ")
-                words.append(splits[0])
-                if len(splits) > 1:
-                    labels.append(splits[-1].replace("\n", ""))
+            if bbox:
+                if line.startswith("-DOCSTART-") or line == "" or line == "\n":
+                    if words:
+                        examples.append(
+                            InputExample(
+                                guid="{}-{}".format(mode, guid_index),
+                                words=words,
+                                labels=labels,
+                                x0=x0,
+                                y0=y0,
+                                x1=x1,
+                                y1=y1,
+                            )
+                        )
+                        guid_index += 1
+                        words = []
+                        labels = []
+                        x0 = []
+                        y0 = []
+                        x1 = []
+                        y1 = []
                 else:
-                    # Examples could have no label for mode = "test"
-                    labels.append("O")
+                    splits = line.split(" ")
+                    words.append(splits[0])
+                    if len(splits) > 1:
+                        labels.append(splits[1].replace("\n", ""))
+                        x0.append(split[2].replace("\n", ""))
+                        y0.append(split[3].replace("\n", ""))
+                        x1.append(split[4].replace("\n", ""))
+                        y1.append(split[5].replace("\n", ""))
+                    else:
+                        # Examples could have no label for mode = "test"
+                        labels.append("O")
+            else:
+                if line.startswith("-DOCSTART-") or line == "" or line == "\n":
+                    if words:
+                        examples.append(
+                            InputExample(guid="{}-{}".format(mode, guid_index), words=words, labels=labels)
+                        )
+                        guid_index += 1
+                        words = []
+                        labels = []
+                else:
+                    splits = line.split(" ")
+                    words.append(splits[0])
+                    if len(splits) > 1:
+                        labels.append(splits[-1].replace("\n", ""))
+                    else:
+                        # Examples could have no label for mode = "test"
+                        labels.append("O")
         if words:
-            examples.append(InputExample(guid="%s-%d".format(mode, guid_index), words=words, labels=labels))
+            if bbox:
+                examples.append(
+                    InputExample(
+                        guid="%s-%d".format(mode, guid_index), words=words, labels=labels, x0=x0, y0=y0, x1=x1, y1=y1
+                    )
+                )
+            else:
+                examples.append(InputExample(guid="%s-%d".format(mode, guid_index), words=words, labels=labels))
     return examples
 
 
-def get_examples_from_df(data):
-    return [
-        InputExample(guid=sentence_id, words=sentence_df["words"].tolist(), labels=sentence_df["labels"].tolist(),)
-        for sentence_id, sentence_df in data.groupby(["sentence_id"])
-    ]
+def get_examples_from_df(data, bbox=False):
+    if bbox:
+        return [
+            InputExample(
+                guid=sentence_id,
+                words=sentence_df["words"].tolist(),
+                labels=sentence_df["labels"].tolist(),
+                x0=sentence_df["x0"].tolist(),
+                y0=sentence_df["y0"].tolist(),
+                x1=sentence_df["x1"].tolist(),
+                y1=sentence_df["y1"].tolist(),
+            )
+            for sentence_id, sentence_df in data.groupby(["sentence_id"])
+        ]
+    else:
+        return [
+            InputExample(guid=sentence_id, words=sentence_df["words"].tolist(), labels=sentence_df["labels"].tolist(),)
+            for sentence_id, sentence_df in data.groupby(["sentence_id"])
+        ]
 
 
 def convert_example_to_feature(example_row):
@@ -111,18 +176,34 @@ def convert_example_to_feature(example_row):
 
     tokens = []
     label_ids = []
-    for word, label in zip(example.words, example.labels):
-        word_tokens = tokenizer.tokenize(word)
-        tokens.extend(word_tokens)
-        # Use the real label id for the first token of the word, and padding ids for the remaining tokens
-        if word_tokens:  # avoid non printable character like '\u200e' which are tokenized as a void token ''
+    bboxes = []
+    if example.bboxes:
+        for word, label, bbox in zip(example.words, example.labels, example.bboxes):
+            word_tokens = tokenizer.tokenize(word)
+            tokens.extend(word_tokens)
+            # Use the real label id for the first token of the word, and padding ids for the remaining tokens
             label_ids.extend([label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1))
+            bboxes.extend([bbox] * len(word_tokens))
+
+        cls_token_box = [0, 0, 0, 0]
+        sep_token_box = [1000, 1000, 1000, 1000]
+        pad_token_box = [0, 0, 0, 0]
+
+    else:
+        for word, label in zip(example.words, example.labels):
+            word_tokens = tokenizer.tokenize(word)
+            tokens.extend(word_tokens)
+            # Use the real label id for the first token of the word, and padding ids for the remaining tokens
+            if word_tokens:  # avoid non printable character like '\u200e' which are tokenized as a void token ''
+                label_ids.extend([label_map[label]] + [pad_token_label_id] * (len(word_tokens) - 1))
 
     # Account for [CLS] and [SEP] with "- 2" and with "- 3" for RoBERTa.
     special_tokens_count = 3 if sep_token_extra else 2
     if len(tokens) > max_seq_length - special_tokens_count:
         tokens = tokens[: (max_seq_length - special_tokens_count)]
         label_ids = label_ids[: (max_seq_length - special_tokens_count)]
+        if bboxes:
+            bboxes = bboxes[: (max_seq_length - special_tokens_count)]
 
     # The convention in BERT is:
     # (a) For sequence pairs:
@@ -144,10 +225,14 @@ def convert_example_to_feature(example_row):
     # the entire model is fine-tuned.
     tokens += [sep_token]
     label_ids += [pad_token_label_id]
+    if bboxes:
+        bboxes += [sep_token_box]
     if sep_token_extra:
         # roberta uses an extra separator b/w pairs of sentences
         tokens += [sep_token]
         label_ids += [pad_token_label_id]
+        if bboxes:
+            bboxes += [sep_token_box]
     segment_ids = [sequence_a_segment_id] * len(tokens)
 
     if cls_token_at_end:
@@ -158,6 +243,8 @@ def convert_example_to_feature(example_row):
         tokens = [cls_token] + tokens
         label_ids = [pad_token_label_id] + label_ids
         segment_ids = [cls_token_segment_id] + segment_ids
+        if bboxes:
+            bboxes = [cls_token_box] + bboxes
 
     input_ids = tokenizer.convert_tokens_to_ids(tokens)
 
@@ -177,13 +264,22 @@ def convert_example_to_feature(example_row):
         input_mask += [0 if mask_padding_with_zero else 1] * padding_length
         segment_ids += [pad_token_segment_id] * padding_length
         label_ids += [pad_token_label_id] * padding_length
+        if bboxes:
+            bboxes += [pad_token_box] * padding_length
 
     assert len(input_ids) == max_seq_length
     assert len(input_mask) == max_seq_length
     assert len(segment_ids) == max_seq_length
     assert len(label_ids) == max_seq_length
+    if bboxes:
+        assert len(bboxes) == max_seq_length
 
-    return InputFeatures(input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_ids=label_ids,)
+    if bboxes:
+        return InputFeatures(
+            input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_ids=label_ids, bboxes=bboxes
+        )
+    else:
+        return InputFeatures(input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_ids=label_ids,)
 
 
 def convert_examples_to_features(
